if(CMAKE_Fortran_COMPILER_ID MATCHES GNU)
  # Fortran language options:
  string(APPEND CMAKE_Fortran_FLAGS " -std=f2003")

  # Error and warning options:
  string(APPEND CMAKE_Fortran_FLAGS
    " -fmax-errors=1 -pedantic -Wall -Wcharacter-truncation -Wunused-parameter"
    " -Wno-conversion -Wimplicit-interface -Wimplicit-procedure"
    " -Wno-integer-division -Wno-maybe-uninitialized")

  # Debugging options:
  set(CMAKE_Fortran_FLAGS_DEBUG "-fbacktrace -g -ffpe-trap=zero,overflow")
  # (We cannot use -ffpe-trap=invalid because of Gfortran bug with
  # quiet NaNs: https://gcc.gnu.org/bugzilla/show_bug.cgi?id=70817.)

  # Code generation options:
  string(APPEND CMAKE_Fortran_FLAGS_DEBUG
    " -fcheck=bounds,do,mem,pointer,recursion -finit-derived -finit-real=snan "
    "-O0")

  # Optimization options:
  set(CMAKE_Fortran_FLAGS_RELEASE -O3)

  # Hardware model options:
  string(APPEND CMAKE_Fortran_FLAGS_RELEASE " -mcmodel=medium")
elseif(CMAKE_Fortran_COMPILER_ID MATCHES Intel)
  # Language:
  string(APPEND CMAKE_Fortran_FLAGS
    " -noaltparam -stand f03 -standard-semantics -assume nostd_mod_proc_name")
  string(APPEND CMAKE_Fortran_FLAGS_DEBUG
    " -check bounds,format,output_conversion,pointers,stack,uninit")

  # Data:

  string(APPEND CMAKE_Fortran_FLAGS " -auto -mcmodel=medium")
  include(CheckFortranCompilerFlag)
  check_fortran_compiler_flag("-init=minus_huge" HAS_FLAG)

  if(HAS_FLAG)
    string(APPEND CMAKE_Fortran_FLAGS_DEBUG " -init=arrays,minus_huge,snan")
  else()
    string(APPEND CMAKE_Fortran_FLAGS_DEBUG " -init=arrays,snan")
  endif()
  
  # Compiler diagnostics:
  string(APPEND CMAKE_Fortran_FLAGS
    " -warn declarations,general,stderrors,truncated_source,uncalled,unused,usage"
    " -traceback -diag-error-limit 1")
  
  # Optimization:
  string(APPEND CMAKE_Fortran_FLAGS_DEBUG " -O0")
  
  # Floating point:
  string(APPEND CMAKE_Fortran_FLAGS_DEBUG " -fp-stack-check -fpe-all=0")
  
  # Debug:
  string(APPEND CMAKE_Fortran_FLAGS_DEBUG
    " -debug full -debug-parameters all -ftrapuv")
elseif(CMAKE_Fortran_COMPILER_ID MATCHES NAG)
  string(APPEND CMAKE_Fortran_FLAGS " -f2003")
  string(APPEND CMAKE_Fortran_FLAGS_DEBUG " -C=all -gline -nan -strict95")
endif()
